import 'package:flutter/cupertino.dart';
import 'package:osam_flutter/osam_flutter.dart';
import 'package:osam_flutter/src/navigation/osam_route.dart';

import 'exceptions.dart';

mixin NavigationPresenter<US, AS, T> on Presenter<US, AS> {
  Navigation<T> get navigation;

  void pop() => navigation.pop();

  void popUntil(T route) => navigation.popUntil(route);

  void setResult<R>({@required T route, R result}) =>
      currentRoutes.firstWhere((element) => element.value == T, orElse: throw NoRouteException()).result = result;

  Future<R> push<A, R>(OsamRoute<T, A, R> route) => navigation.push(route);

  Future<R> pushReplacement<A, R>(OsamRoute<T, A, R> route) => navigation.pushReplacement(route);

  Future<R> pushAndRemoveUntil<A, R>(OsamRoute<T, A, R> route) => navigation.pushAndRemoveUntil(route);

  List<OsamRoute<T, Object, Object>> get currentRoutes => navigation.routes.value;

  OsamRoute<T, Object, Object> get currentRoute => currentRoutes.last;

  OsamRoute<T, Object, Object> get firstRoute => currentRoutes.first;

  bool isInStack(T route) => currentRoutes.contains(route);

  bool get canPop => navigation.routes.value.length > 1;

  OsamRoute<T, Object, Object> get routeBefore =>
      currentRoutes.length > 1 ? currentRoutes[currentRoutes.length - 2] : throw NoRouteException();
}

mixin TabNavigationPresenter<US, AS, T> on Presenter<US, AS> {
  TabNavigation get tabNavigation;

  String get currentTab => tabNavigation.tabNotifier.value;

  void changeTab(T newTab) => tabNavigation.tab = newTab;
}
