import 'package:osam/osam.dart';
import 'package:osam_flutter/src/navigation/osam_route.dart';

mixin Navigation<T> {
  PropertyNotifier<List<OsamRoute<T, Object, Object>>> get routes;

  Future<R> push<A, R>(OsamRoute<T, A, R> route) {
    routes.update((value) => value.add(route));
    return route.completer.future;
  }

  void pop() => routes.update((value) => value.removeLast());

  Future<R> pushReplacement<A, R>(OsamRoute<T, A, R> route) {
    routes.update((value) {
      value.removeLast();
      value.add(route);
    });
    return route.completer.future;
  }

  Future<R> pushAndRemoveUntil<A, R>(OsamRoute<T, A, R> route) {
    routes.update((value) {
      value.clear();
      value.add(route);
    });
    return route.completer.future;
  }

  void popUntil(T route) => routes.update((value) {
        if (value.contains(route)) {
          while (value.last != route) {
            value.removeLast();
          }
        }
      });
}

mixin TabNavigation<T> {
  PropertyNotifier<T> get tabNotifier;

  set tab(T tab) => tabNotifier.value = tab;
}
